/// CaveGame - A procedural 2D platformer sandbox.
/// Created by Josh O'Leary @ Redacted Software, 2020-2025
/// Contact: josh@redacted.cc
/// Contributors: william@redacted.cc maxi@redacted.cc
/// This work is dedicated to the public domain.

/// @file Console.hpp
/// @desc The client-side user interface for the command console.
/// @edit 1/28/2025
/// @auth Josh O'Leary

#pragma once

#include <JUI/Widgets/Window.hpp>
#include <JUI/Widgets/TextInputForm.hpp>
#include <JUI/Widgets/ScrollingRect.hpp>
#include <JUI/Widgets/ListLayout.hpp>

namespace CaveGame::Client {
    using namespace JUI::UDimLiterals;
    class Console : public JUI::Window {
    public:

        int index = 0;

        Event<std::string> OnInput;


        void OnInputEvent(const std::string& msg);

        explicit Console(Widget* parent);

        void Log(const std::string& message, const Color4& color = Colors::White);

        void ObserveKeyInput(Key key, bool pressed) override;
        bool IsOpen() const;
        void SetOpen(bool open);
    protected:
        JUI::TextInputForm* input_box;
        JUI::ScrollingRect* message_log_box;
        JUI::VerticalListLayout* message_log_layout;
        bool console_open = false;
        std::vector<std::string> history;
    private:
    };
}