#include <Client/Console.hpp>
#include <Core/Loggers.hpp>

void CaveGame::Client::Console::OnInputEvent(const std::string &msg) {
    Log(std::format(">{}", msg));
    OnInput.Invoke(msg);
    // TODO: Handle within JUI
    input_box->SetAutoCompleteText("");
    input_box->SetContent("");
}

CaveGame::Client::Console::Console(JUI::Widget *parent) : JUI::Window(parent)
{
    this->SetTitle("Console");

    message_log_box = new JUI::ScrollingRect(this->ViewportInstance());
    message_log_box->Size(JUI::UDim2(0, -20, 1, 1));

    message_log_layout = new JUI::VerticalListLayout(message_log_box);
    message_log_layout->LayoutOrder(JUI::LayoutOrder::V::BOTTOM);

    //message_log_view->Position(JUI::UDim2());

    input_box = new JUI::TextInputForm(this->ViewportInstance());
    input_box->SetContent("Test");
    input_box->Size(JUI::UDim2(0, 20, 1, 0));
    input_box->Position(JUI::UDim2(0, -20, 0, 1));
    input_box->BGColor(Colors::Grays::DarkSlateGray);
    input_box->SetTextSize(16);
    input_box->SetAutoCompleteText("");
    input_box->SetTextColor(Colors::White);
    input_box->SetAutocompleteTextColor(Colors::White);
    // TODO: input_box->ClearTextOnReturn(true);

    input_box->OnReturn += [this] (const std::string& msg) {
        OnInputEvent(msg);
    };

    CaveGame::Logs::Info.OnLog += [this](std::string m, Color4 c) {Log(m, c); };
    CaveGame::Logs::Debug.OnLog += [this](std::string m, Color4 c) {Log(m, c); };
    CaveGame::Logs::Warning.OnLog += [this](std::string m, Color4 c) {Log(m, c); };
    CaveGame::Logs::Error.OnLog += [this](std::string m, Color4 c) {Log(m, c); };
    CaveGame::Logs::Fatal.OnLog += [this](std::string m, Color4 c) {Log(m, c); };

    CaveGame::Logs::Client.OnLog += [this](std::string m, Color4 c) {Log(m, c); };
    CaveGame::Logs::Server.OnLog += [this](std::string m, Color4 c) {Log(m, c); };
    CaveGame::Logs::Generator.OnLog += [this](std::string m, Color4 c) {Log(m, c); };
    CaveGame::Logs::Lighting.OnLog += [this](std::string m, Color4 c) {Log(m, c); };

    // TODO: Integrate loggers into game console.
}

void CaveGame::Client::Console::Log(const std::string &message, const Color4& color) {

    Color4 col_a = {64, 64, 64};
    Color4 col_b = {92, 92, 92};

    auto* entry = new JUI::TextRect(message_log_layout);
    entry->Size({100_percent, 16_px});
    entry->SetContent(message);
    entry->LayoutOrder(index++);
    entry->SetTextColor(Colors::Black);
    entry->BGColor(index%2==0 ? col_a : col_b);
    entry->SetTextSize(14);
    entry->SetTextColor(color);
}

void CaveGame::Client::Console::ObserveKeyInput(Key key, bool pressed) {
    if (!IsVisible())
        return;

    Widget::ObserveKeyInput(key, pressed);
}

void CaveGame::Client::Console::SetOpen(bool open) {
    console_open = open;

    if (open)
        input_box->GrabFocus();
    else
        input_box->DropFocus();

    Visible(open);

}

bool CaveGame::Client::Console::IsOpen() const { return console_open;}
